

import java.io.*;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import topology.*;
import homology.*;
import homology.f2homology.*;

public class CityGMLDemo {
    
    public static void main(String args[]) {

	try {
	    
	    Timer timer = new Timer();
	    
	    // data
	    File house = new File("data/tetrahedron.xml");
	    //File house = new File("data/tetrahedron+hole.xml");
	    //File house = new File("data/meinhaus.xml");
	    //File house = new File("data/meinehaeuser.xml");
	    //File house = new File("data/surface.xml");
	    //File house = new File("data/window.xml");
	    //File house = new File("data/wallSurface.xml");
	    //File house = new File("data/cityObjectMember.xml");
	    	    
	    // parsing
	    timer.start();
	    System.out.println("parsing...");
	    CityGMLDomParser parser = new CityGMLDomParser(house);
	    timer.end();
	    long totalTime = timer.getTotalTime();
	    System.out.println("Time: " + timer.timeFormat(totalTime));
	    System.out.println();
	    
	    // compute space
	    timer.start();
	    System.out.println("computing finite space");
	    FiniteSpace<String> space = parser.space();
	    //System.out.println("topological relation: ");
	    //System.out.println(space.getRelation().getPairs());
	    // System.out.println("Vertices:"); 
	    // System.out.println(parser.getVertices());
	    // System.out.println();
	    // System.out.println("Lines:"); 
	    // System.out.println(parser.getLines());
	    // System.out.println();
	    // System.out.println("Faces:"); 
	    // System.out.println(parser.getFaces());
	    // System.out.println();
	    // System.out.println("extra faces:");
	    // System.out.println(parser.getExtraFaces());
	    System.out.print("point space: " + space.getPointSet().size()); 
	    System.out.println(" points = " + parser.getVertices().size() 
	    		       + " vertices + " + parser.getLines().size() + " lines + " 
	    		       + parser.getFaces().size() + " faces + "
			       + parser.getExtraFaces().size() + " extra faces");
	    System.out.println("relation: " + space.getRelation().getPairs().size() + " pairs");
	   timer.end();
	   totalTime = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTime));
	   System.out.println();

	   // spanning tree
	   timer.start();
	   System.out.println("Method 1.");
	   FiniteSpace<String> pureSpace = new FiniteSpace(space);
	   pureSpace.removeAllPoint(parser.getExtraFaces());
	   System.out.println("Spanning tree of pure space...");
	   PrimMST mst = new PrimMST(pureSpace);
	   System.out.print("Pair difference: ");
	   int pairDiff = pureSpace.getRelation().getPairs().size() 
	       - mst.getMstRelation().getPairs().size();
	   System.out.println(pairDiff);
	   timer.end();
	   long totalTimeMethod1_1 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod1_1));
	   System.out.println(); 
	   
	   // quotient complex of pure space
	   timer.start();
	   System.out.println("quotient complex of pure space...");
	   FiniteSpace mstSpace 
	       = new FiniteSpace(pureSpace.getPointSet(),mst.getMstRelation());
	   F2OrderComplexQuotient pureQuotient = 
	       new F2OrderComplexQuotient(mstSpace,pureSpace);
	   timer.end();
	   long totalTimeMethod1_2 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod1_2));
	   System.out.println();

	   // Betti numbers via quotient of pure space
	   timer.start();
	   System.out.println("Betti numbers via quotient of pure space...");
	   F2ChainComplex pureQuotientComplex = pureQuotient
	       .getF2ChainComplex();
	   int[] pureBetti = pureQuotientComplex.BettiNumbers();
	   System.out.println("b0 = " + (mst.componentNumber()));
	   for (int i = 1; i < pureBetti.length; i++) {
	       System.out.println("b" + i + " = " + pureBetti[i]);
	   }
	   timer.end();
	   long totalTimeMethod1_3 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod1_3) );
	   System.out.println("Total Time Method 1: " 
			      + timer.timeFormat(totalTimeMethod1_1 + totalTimeMethod1_2 
				 + totalTimeMethod1_3));
	   System.out.println();
	   
	   // quasi-core
	   timer.start();
	   System.out.println("Method 2.");
	   System.out.println("quasi-core of pure space...");
	   FiniteSpace<String> pureQuasiCore = parser.pureQuasiCore(); // this is faster!
	   //	       = PointReductions.removeWeakPoints(pureSpace); // this is smaller but slower!
	   System.out.println(pureQuasiCore.getPointSet().size() + " point(s)");
	   System.out.println(pureQuasiCore.getRelation().getPairs().size() 
			      + " pair(s)");
	   //System.out.println(pureQuasiCore.getRelation().getPairs());
	   //System.out.println("1-core dimension = " + pureQuasiCore.dimension());
	   timer.end();
	   long totalTimeMethod2_1 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod2_1));
	   System.out.println();
	   
	   // spanning tree of quasi-core
	   timer.start();
	   System.out.println("Spanning tree of quasi-core...");
	   PrimMST mstCore = new PrimMST(pureQuasiCore);
	   System.out.print("Pair difference: ");
	   pairDiff = pureQuasiCore.getRelation().getPairs().size() 
	       - mstCore.getMstRelation().getPairs().size();
	   System.out.println(pairDiff);
	   timer.end();
	   long totalTimeMethod2_2 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod2_2));
	   System.out.println();
	    
	   // quotient complex of quasi-core
	   timer.start();
	   System.out.println("quotient complex of quasi-core...");
	   FiniteSpace mstCoreSpace 
	       = new FiniteSpace(pureQuasiCore.getPointSet(),
				 mstCore.getMstRelation());
	   F2OrderComplexQuotient quotient 
	       = new F2OrderComplexQuotient(mstCoreSpace,pureQuasiCore); 
	   timer.end();
	   long totalTimeMethod2_3 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod2_3));
	   System.out.println();

	   // Betti numbers via quotient complex of quasi-core
	   timer.start();
	   System.out.println("Betti numbers via quotient complex of quasi-core...");
	   F2ChainComplex quotientComplex = quotient.getF2ChainComplex();
	   int[] qBetti = quotientComplex.BettiNumbers();
	   int quotientEc = parser.EulerCharacteristic(quotientComplex);
	   System.out.println("b0 = " + (mstCore.componentNumber()));
	   for (int i = 1; i < qBetti.length; i++) {
	       if (i == 2) {
		   int qBetti2 = quotientEc + qBetti[1];
		   System.out.println("b2 = " + qBetti2);
	       } else {
		   System.out.println("b" + i + " = " + qBetti[i]);
	       }
	   }
	   for (int i = pureQuasiCore.dimension() + 1; i < space.dimension() + 1; i++) {
	       System.out.println("b" + i + " = 0");
	   }
	   timer.end();
	   long totalTimeMethod2_4 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod2_4));
	   System.out.println("Total time Method 2: " 
			      + timer.timeFormat(totalTimeMethod2_1 + totalTimeMethod2_2
				 + totalTimeMethod2_3 + totalTimeMethod2_4));
	   System.out.println();

	   // Betti numbers of full space via quotient complex of 1-core
	   timer.start();
	   System.out.println("Method 3.");
	   System.out.println("Betti numbers of full space via quotient complex of quasi-core:");
	   System.out.println("quasi-core...");
	   FiniteSpace<String> quasiCore = parser.quasiCore(); // this is faster
	   // = PointReductions.removeWeakPoints(space);          // than this
	   System.out.println(quasiCore.getPointSet().size() + " points");
	   System.out.println(quasiCore.getRelation().getPairs().size() + " pairs");
	   System.out.println("Spanning tree...");
	   PrimMST mst1Core = new PrimMST(quasiCore);
	   System.out.println("quotient complex...");
	   FiniteSpace mst1CoreSpace 
	       = new FiniteSpace(quasiCore.getPointSet(),
				 mst1Core.getMstRelation());
	   F2OrderComplexQuotient coreQuotient 
	       = new F2OrderComplexQuotient(mst1CoreSpace,quasiCore); 
	   System.out.println("Betti numbers: b2 = b2 of pure space + extra openings...");
	   F2ChainComplex coreQuotientComplex 
	       = coreQuotient.getF2ChainComplex();
	   int[] coreQBetti = coreQuotientComplex.BettiNumbers();
	   int coreQuotientEc = parser.EulerCharacteristic(coreQuotientComplex);
	   System.out.println("b0 = " + (mst1Core.componentNumber()));
	   for (int i = 1; i < coreQBetti.length; i++) {
	       if (i == 2) {
	         int coreQBetti2 = coreQuotientEc + coreQBetti[1];
	         System.out.println("b2 = " + coreQBetti2);
	       } else {
		   System.out.println("b" + i + " = " + coreQBetti[i]);
	       }
	   }
	   for (int i = quasiCore.dimension() + 1; i < space.dimension() + 1; i++) {
	       System.out.println("b" + i + " = 0");
	   }
	   System.out.println("extra openings: " + parser.getExtraFaces().size());
	   timer.end();
	   long totalTimeMethod3 = timer.getTotalTime();
	   System.out.println("Total time Method 3: " + timer.timeFormat(totalTimeMethod3));
	   System.out.println();
	   
	   // Betti numbers via Order complex of quasi-core
	   timer.start();
	   System.out.println("Method 4.");
	   System.out.println("Betti numbers via order complex of quasi-core...");
	   Betti betti = new Betti(pureQuasiCore);
	   for (int i = 0; i < pureQuasiCore.dimension() + 1; i++) {
	   	   System.out.println("b" + i + " = " 
				      + betti.F2BettiNumbers()[i]);
	   }
	   for (int i = pureQuasiCore.dimension() + 1; i < space.dimension() + 1; 
		i++) {
	       System.out.println("b" + i + " = 0");
	   }
	   timer.end();
	   long totalTimeMethod4 = timer.getTotalTime();
	   System.out.println("Time: " + timer.timeFormat(totalTimeMethod4));
	   System.out.println("Total time Method 4: " + timer.timeFormat(totalTimeMethod4 + totalTimeMethod2_1));
	   System.out.println();

	   // Betti numbers via Order complex of pure space
	   timer.start();
	   System.out.println("Method 5.");
	   System.out.println("Betti numbers via order complex of pure space...");
	   Betti pureSpaceBetti = new Betti(pureSpace);
	   for (int i = 0; i < pureSpace.dimension() + 1; i++) {
	 	   System.out.println("b" + i + " = " 
				      + pureSpaceBetti.F2BettiNumbers()[i]);
	   }
	   timer.end();
	   long totalTimeMethod5 = timer.getTotalTime();
	   System.out.println("Total time Method 5: " + timer.timeFormat(totalTimeMethod5));

	   System.out.println("End.");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
    
}
